/// <reference path="../typings/tsd.d.ts" />
/// <reference path="../node_modules/typescript/lib/typescript.d.ts" />
/// <reference path="../node_modules/immutable/dist/immutable.d.ts"/>
var ts = require("typescript");
var path = require("path");
var _ = require('lodash');
var fs_1 = require('fs');
var formatter_1 = require('./formatter');
var util_1 = require('./util');
var ejs = require('ejs');
function generate(fileName, text, extOptions) {
    var sourceFile = ts.createSourceFile(fileName, text.toString(), 2, false);
    var templatesFolder = path.join(__dirname, '../templates');
    var _a = util_1.walker(sourceFile), ifaces = _a[0], imports = _a[1];
    var template = fs_1.readFileSync(path.join(templatesFolder, 'file.ejs')).toString();
    var importName = path.basename(fileName.replace(/.tsx?$/, ''));
    var flatImports = {};
    imports.forEach(function (imp) {
        if (imp.importClause && imp.importClause.namedBindings) {
            imp.importClause.namedBindings.elements.forEach(function (el) {
                flatImports[el.name.text] = true;
            });
        }
    });
    var options = {
        ifaces: ifaces,
        importName: importName,
        flatImports: flatImports,
        keyType: "string",
        indexerType: 'any',
        emitMaps: true,
        emitRecords: false,
        emitMarkers: false,
        emitEmptyRecords: false,
        emitTypedMethods: false,
        defaultEmptyType: 'null',
        mapSuffix: 'Map',
        recordSuffix: 'Record',
        dictType: 'Dictionary',
        exportDeps: []
    };
    _.assign(options, extOptions);
    var functions = {
        type: function (member) {
            return sourceFile.text.slice(member.type.pos + 1, member.type.end);
        },
        isLocalType: function (typeName) {
            return _.any(ifaces, function (iface) { return iface.name.text == typeName; });
        },
        isArrayType: function (typeName) {
            return typeName.indexOf('[]') !== -1;
        },
        isDictionaryType: function (typeName) {
            return typeName.indexOf(options.dictType) !== -1;
        },
        sanitizeTypeName: function (typeName) {
            var arrayRegex = /\[\]$/;
            if (typeName.indexOf('<') !== -1) {
                return typeName.match(/<(.*?)>/)[1].replace(arrayRegex, '');
            }
            else {
                return typeName.replace(arrayRegex, '');
            }
        },
        hasMapPairType: function (typeName) {
            return flatImports[typeName + options.mapSuffix];
        },
        hasRecordPairType: function (typeName) {
            return flatImports[typeName + options.recordSuffix];
        },
        getImportedImmutableTypes: function () {
            return Object.keys(flatImports).filter(function (item) { return /^[A-Z].*Map$/.test(item); }).map(function (item) { return item.replace(/Map$/, ''); });
        },
        typeMap: function (member, isRecord) {
            if (isRecord === void 0) { isRecord = false; }
            var typeName = functions.type(member);
            var sanitizedTypeName = functions.sanitizeTypeName(typeName);
            var isArrayType = functions.isArrayType(typeName);
            var isDictionaryType = functions.isDictionaryType(typeName);
            var isLocalType = functions.isLocalType(sanitizedTypeName);
            var hasMapPairType = functions.hasMapPairType(sanitizedTypeName);
            var hasRecordPairType = functions.hasRecordPairType(sanitizedTypeName);
            if (isLocalType || hasMapPairType || hasRecordPairType) {
                var suffix = (isRecord && (isLocalType || hasRecordPairType))
                    ? options.recordSuffix
                    : options.mapSuffix;
                if (isArrayType) {
                    return "Immutable.List<" + sanitizedTypeName + suffix + ">";
                }
                else if (isDictionaryType) {
                    return "Immutable.Map<string, " + sanitizedTypeName + suffix + ">";
                }
                else {
                    return "" + typeName + suffix;
                }
            }
            else {
                if (isArrayType) {
                    return "Immutable.List<" + sanitizedTypeName + ">";
                }
                else {
                    return "" + typeName;
                }
            }
        },
        deps: function (ifaceName, acc) {
            if (acc === void 0) { acc = {}; }
            var iface = _.find(ifaces, function (iface) { return iface.name.text == ifaceName; });
            iface.members.forEach(function (member) {
                var memberTypeName = functions.type(member);
                var sanitizedTypeName = functions.sanitizeTypeName(memberTypeName);
                var isVisited = acc[sanitizedTypeName];
                if (!isVisited) {
                    if (functions.isLocalType(sanitizedTypeName)) {
                        acc[sanitizedTypeName] = true;
                        functions.deps(sanitizedTypeName, acc);
                    }
                }
            });
            acc[ifaceName] = true;
            return Object.keys(acc);
        },
        ownDeps: function (ifaceName, acc) {
            if (acc === void 0) { acc = {}; }
            var iface = _.find(ifaces, function (iface) { return iface.name.text == ifaceName; });
            iface.members.forEach(function (member) {
                var memberTypeName = functions.type(member);
                var sanitizedTypeName = functions.sanitizeTypeName(memberTypeName);
                if (functions.isLocalType(sanitizedTypeName)
                    || functions.hasMapPairType(sanitizedTypeName)
                    || functions.hasRecordPairType(sanitizedTypeName)) {
                    acc[member.name.text] = memberTypeName;
                }
            });
            return acc;
        },
        initializer: function (member) {
            var typeName = functions.type(member);
            var sanitizedTypeName = functions.sanitizeTypeName(typeName);
            var isArrayType = functions.isArrayType(typeName);
            var isDictionaryType = functions.isDictionaryType(typeName);
            var isLocalType = functions.isLocalType(sanitizedTypeName);
            var hasRecordPairType = functions.hasRecordPairType(sanitizedTypeName);
            var suffix = (isLocalType || hasRecordPairType)
                ? options.recordSuffix
                : options.mapSuffix;
            if (isLocalType) {
                if (isArrayType) {
                    return "Immutable.List<" + sanitizedTypeName + suffix + ">()";
                }
                else if (isDictionaryType) {
                    return "Immutable.Map<string," + sanitizedTypeName + suffix + ">()";
                }
                else if (!member.questionToken) {
                    return "new " + typeName + suffix + "()";
                }
                else {
                    return options.defaultEmptyType;
                }
            }
            else {
                return options.defaultEmptyType;
            }
        },
        exportDeps: function () {
            var deps = {};
            ifaces.forEach(function (iface) {
                deps[iface.name.text] = true;
                iface.members.forEach(function (member) {
                    var memberTypeName = functions.type(member);
                    var sanitizedTypeName = functions.sanitizeTypeName(memberTypeName);
                    if (!functions.isLocalType(sanitizedTypeName)) {
                        var internalTypes = functions.extractTypes(memberTypeName);
                        internalTypes.forEach(function (type) {
                            deps[type] = true;
                            if (functions.hasMapPairType(type)) {
                                deps[type + options.mapSuffix] = true;
                            }
                            if (functions.hasRecordPairType(type)) {
                                deps[type + options.recordSuffix] = true;
                            }
                        });
                    }
                });
            });
            return Object.keys(deps);
        },
        extractTypes: function (typeName) {
            var types = typeName
                .match(/(?:[^a-z]|^)([A-Z]\w+)/g);
            if (types) {
                return types.map(function (type) { return type.match(/[A-Z]\w+/); }).map(function (s) { return s.toString(); });
            }
            else {
                return [];
            }
        },
        forEach: _.forEach
    };
    options.exportDeps = functions.exportDeps();
    var result = ejs.render(template, options, {
        filename: path.join(templatesFolder, 'file.ejs'),
        context: functions
    });
    return formatter_1.format(result);
}
exports.generate = generate;
