/// <reference path="../../typings/tsd.d.ts" />
var index_1 = require('../index');
var fs_1 = require('fs');
var chai_1 = require('chai');
var _ = require('lodash');
var path = require('path');
var MODELS_FOLDER = path.join(process.cwd(), 'src', 'spec', 'models');
var Modules = {
    User: path.join(MODELS_FOLDER, 'user.ts'),
    Profile: path.join(MODELS_FOLDER, 'profile.ts'),
    Acl: path.join(MODELS_FOLDER, 'acl.ts')
};
describe('Scope', function () {
    var scope;
    beforeEach(function () {
        scope = new index_1.Scope({});
        _.forEach(Modules, function (modPath, modName) {
            scope.addModule(modPath, fs_1.readFileSync(modPath).toString());
            var mod = scope.getModule(modPath);
            chai_1.expect(mod).to.exist;
            chai_1.expect(mod.fileName).to.equal(mod.fileName);
            chai_1.expect(mod.sourceFile).to.exist;
        });
    });
    it('should contain modules', function () {
        chai_1.expect(Object.keys(scope.modules)).to.be.lengthOf(3);
    });
    describe(Modules.User, function () {
        var mod;
        beforeEach(function () {
            mod = scope.getModule(Modules.User);
        });
        it('should parse', function () {
            chai_1.expect(mod.ifaces).to.be.lengthOf(1);
            chai_1.expect(mod.flatImports).to.be.deep.equal({
                Profile: './profile'
            });
            chai_1.expect(mod.namespaceImports).to.be.deep.equal({
                acl: './acl'
            });
        });
        it('should classify local interfaces', function () {
            chai_1.expect(mod.localInterface('User')).to.deep.equal({ modulePath: null, typeName: 'User' });
            chai_1.expect(mod.localInterface('Profile')).to.be.not.ok;
        });
        it('should classify Profile imported type', function () {
            var modInfo = mod.importedType('Profile');
            chai_1.expect(modInfo).to.be.ok;
            chai_1.expect(modInfo.typeName).to.equal('Profile');
            chai_1.expect(modInfo.modulePath).to.equal('./profile');
        });
        it('should classify Acl imported type', function () {
            var modInfo = mod.importedType('acl.Acl');
            chai_1.expect(modInfo).to.be.ok;
            chai_1.expect(modInfo.typeName).to.equal('Acl');
            chai_1.expect(modInfo.modulePath).to.equal('./acl');
        });
        it('should classify Profile as external interface', function () {
            var modInfo = mod.externalInterface('Profile');
            chai_1.expect(modInfo).to.be.ok;
            chai_1.expect(modInfo.typeName).to.equal('Profile');
            chai_1.expect(modInfo.modulePath).to.equal('./profile');
        });
        it('should classify Acl as external interface', function () {
            var modInfo = mod.externalInterface('acl.Acl');
            chai_1.expect(modInfo).to.be.ok;
            chai_1.expect(modInfo.typeName).to.equal('Acl');
            chai_1.expect(modInfo.modulePath).to.equal('./acl');
        });
    });
    describe(Modules.Profile, function () {
        var mod;
        beforeEach(function () {
            mod = scope.getModule(Modules.Profile);
        });
        it('should parse', function () {
            var mod = scope.getModule(Modules.Profile);
            chai_1.expect(mod.ifaces).to.be.lengthOf(2);
            chai_1.expect(mod.flatImports).to.be.deep.equal({ Theme: './enums' });
        });
    });
});
